# -*- coding: utf-8 -*-
"""
Created on Sat Mar  1 08:38:17 2025

@author: mstep
"""

import tkinter as tk
from tkinter import ttk
from tkinter import messagebox
from tkinter import filedialog as fd
from PIL import Image, ImageTk
import pathlib

class OverlayWMaskApp:
    def __init__(self):
        self.background_image = None
        self.overlay_image = None
        self.mask_image = None
        self.result_image = None
        self.photo_images = {}

        self.window = tk.Tk()

        self.window.title('pil_overlay_with_mask')
        self.window.protocol('WM_DELETE_WINDOW', self.kill_callback)
        self.window.geometry('530x430')

        # Build the menu.
        self.build_menus()

        # Create the tabs.
        self.notebook = ttk.Notebook(self.window)
        self.notebook.pack(side=tk.TOP, expand=True, fill=tk.BOTH)

        # Background image
        self.background_tab = ttk.Frame(self.notebook)
        self.notebook.add(self.background_tab, text='  Background  ')
        self.background_canvas = tk.Canvas(self.background_tab, bg='white',
            borderwidth=0, highlightthickness=0)
        self.background_canvas.pack(side=tk.TOP, expand=True, fill=tk.BOTH,
            padx=10, pady=10)

        # Overlay image
        self.overlay_tab = ttk.Frame(self.notebook)
        self.notebook.add(self.overlay_tab, text='  Overlay  ')
        self.overlay_canvas = tk.Canvas(self.overlay_tab, bg='white',
            borderwidth=0, highlightthickness=0)
        self.overlay_canvas.pack(side=tk.TOP, expand=True, fill=tk.BOTH,
            padx=10, pady=10)

        # Mask image
        self.mask_tab = ttk.Frame(self.notebook)
        self.notebook.add(self.mask_tab, text='  Mask  ')
        self.mask_canvas = tk.Canvas(self.mask_tab, bg='white',
            borderwidth=0, highlightthickness=0)
        self.mask_canvas.pack(side=tk.TOP, expand=True, fill=tk.BOTH,
            padx=10, pady=10)

        # Result image
        self.result_tab = ttk.Frame(self.notebook)
        self.notebook.add(self.result_tab, text='  Result  ')
        self.result_canvas = tk.Canvas(self.result_tab, bg='white',
            borderwidth=0, highlightthickness=0)
        self.result_canvas.pack(side=tk.TOP, expand=True, fill=tk.BOTH,
            padx=10, pady=10)

        # Display the window.
        self.window.focus_force()
        self.window.mainloop()

    def build_menus(self):
        '''Build the menu bar.'''
        menu_bar = tk.Menu(self.window)
        self.window.config(menu=menu_bar)
        self.build_file_menu(menu_bar)

    def build_file_menu(self, menu_bar):
        '''Build the File menu.'''
        self.file_menu = tk.Menu(menu_bar, tearoff=False)
        menu_bar.add_cascade(label='File', menu=self.file_menu)

        # Open main
        self.open_photo_image = ImageTk.PhotoImage(file='open.png')
        self.file_menu.add_command(label='Background Image...', accelerator='Ctrl+B',
            compound='left',
            command=self.mnu_open_background, image=self.open_photo_image)
        self.window.bind('<Control-b>', lambda event: self.mnu_open_background())

        # Open overlay
        self.file_menu.add_command(label='Overlay Image...', accelerator='Ctrl+O',
            compound='left',
            command=self.mnu_open_overlay, image=self.open_photo_image)
        self.window.bind('<Control-o>', lambda event: self.mnu_open_overlay())

        # Open mask
        self.file_menu.add_command(label='Mask Image...', accelerator='Ctrl+M',
            compound='left',
            command=self.mnu_open_mask, image=self.open_photo_image)
        self.window.bind('<Control-m>', lambda event: self.mnu_open_mask())

        # Save
        self.file_menu.add_command(label='Save As...', accelerator='Ctrl+S',
            compound='left',
            command=self.mnu_save_as, image=self.open_photo_image,
            state='disabled')
        self.window.bind('<Control-s>', lambda event: self.mnu_save_as())

        self.file_menu.add_separator()

        # Exit
        self.exit_photo_image = ImageTk.PhotoImage(file='exit.png')
        self.file_menu.add_command(label='Exit', compound='left',
            command=self.mnu_exit, image=self.exit_photo_image)

    def mnu_open_background(self):
        image = self.get_image_file('Background Image', self.background_canvas)
        if image is None: return
        self.background_image = image
        self.notebook.select(self.background_tab)
        # Perform the overlay (if we have all three images).
        self.perform_overlay()

    def mnu_open_overlay(self):
        image = self.get_image_file('Overlay Image', self.overlay_canvas)
        if image is None: return
        self.overlay_image = image
        self.notebook.select(self.overlay_tab)
        # Perform the overlay (if we have all three images).
        self.perform_overlay()

    def mnu_open_mask(self):
        image = self.get_image_file('Mask Image', self.mask_canvas)
        if image is None: return
        self.mask_image = image
        self.notebook.select(self.mask_tab)
        # Perform the overlay (if we have all three images).
        self.perform_overlay()

    def mnu_save_as(self):
        self.save_image_as(self.result_image)

    def save_image_as(self, image):
        '''Save a file.'''
        filetypes = (
            ('Image Files', '*.png;*.jpg;*.bmp;*.gif;*.tif'),
            ('All files', '*.*')
        )
        filename = fd.asksaveasfilename(filetypes=filetypes,
            defaultextension='.jpg')
        if len(filename) == 0:
            return

        # See what kind of file is it.
        suffix = pathlib.Path(filename).suffix.lower()
        if suffix == '.jpg' or suffix == '.jpeg':
            # It's a JPG. Save in RGB format.
            image.convert('RGB').save(filename)
        else:
            # Save in RGBA format.
            image.save(filename)

    def get_image_file(self, title, canvas):
        '''
        Let the user pick an image file.
        Then load and return it in an RGB PIL Image.
        '''
        filetypes = (
            ('Image Files', '*.png;*.jpg;*.bmp;*.gif;*.tif'),
            ('All files', '*.*')
        )
        filename = fd.askopenfilename(title=title, filetypes=filetypes)
        if not filename:
            return None

        try:
            image = Image.open(filename)
            self.show_image(image, canvas)
            return image
        except Exception as e:
            messagebox.showinfo('Open Error', f'Error loading image file.\n{e}')
            return None

    def show_image(self, image, canvas):
        '''Display the image on the canvas.'''
        # Convert into a PhotoImage.
        photo_image = ImageTk.PhotoImage(image)

        # Save a non-volatile copy.
        self.photo_images[canvas] = photo_image

        # Display it.
        canvas.delete(tk.ALL)
        canvas.create_image(0, 0, anchor=tk.NW, image=photo_image)
        wid = photo_image.width()
        hgt = photo_image.height()
        canvas.config(width=wid, height=hgt)

    def perform_overlay(self):
        '''Perform the overlay.'''
        # See if we have all three images.
        if self.background_image is None or \
           self.overlay_image is None or \
           self.mask_image is None:
                return

        # Enable the Save menu item.
        self.file_menu.entryconfig('Save As...', state='normal')

        # Convert the background and overlay images to RGBA.
        background = self.background_image.convert('RGBA')
        overlay = self.overlay_image.convert('RGBA')

        # Convert mask to grayscale.
        mask = self.mask_image.convert('L')

        # Perform the overlay.       
        background.paste(overlay, (0, 0), mask)

        # Save and display the result.
        self.result_image = background
        self.show_image(self.result_image, self.result_canvas)

        # Select the result tab.
        self.notebook.select(self.result_tab)

    def mnu_exit(self):
        print('Exit')
        self.kill_callback()

    def kill_callback(self):
        # Destroy the tkinter window.
        self.window.destroy()

# In[19]:
OverlayWMaskApp()
print('Done')
